# BBB

from zope.component import getAdapters
from zope.interface import implements
from zope.app.component.hooks import getSite

try:
    try:
        from Products.PortalTransforms.interfaces import ITransform
    except ImportError:
        from Products.PortalTransforms.z3.interfaces import ITransform
except ImportError:
    ITransform = None
from Products.PortalTransforms.interfaces import itransform

from plone.outputfilters.interfaces import IFilter
from plone.outputfilters import apply_filters


class html_to_tinymce_output_html:
    """ transform which converts html to tiny output html"""
    if ITransform is not None:
        implements(ITransform)
    __implements__ = itransform
    __name__ = "html_to_tinymce_output_html"
    inputs = ('text/html',)
    output = "text/x-tinymce-output-html"

    def __init__(self, name=None):
        self.config_metadata = {
            'inputs' : ('list', 'Inputs', 'Input(s) MIME type. Change with care.'),
        }
        if name:
            self.__name__ = name

    def name(self):
        return self.__name__

    def convert(self, orig, data, **kwargs):
        """apply plone.outputfilters filters"""
        context = kwargs.get('context')
        request = getattr(getSite(), 'REQUEST', None)
        filters = [f for _, f in getAdapters((context, request), IFilter)]
        
        res = apply_filters(filters, orig)
        data.setData(res)
        return data

# This needs to be here to avoid breaking existing instances
def register():
    return html_to_tinymce_output_html()



import httplib

from zope.interface import implements
from zope.component import getUtility
from AccessControl import ClassSecurityInfo

from Products.Five.browser import BrowserView

from Products.TinyMCE.adapters.interfaces.JSONFolderListing import IJSONFolderListing
from Products.TinyMCE.adapters.interfaces.JSONSearch import IJSONSearch
from Products.TinyMCE.adapters.interfaces.JSONDetails import IJSONDetails
from Products.TinyMCE.adapters.interfaces.Upload import IUpload
from Products.TinyMCE.adapters.interfaces.Save import ISave
from Products.TinyMCE.browser.interfaces.browser import ITinyMCEBrowserView
from Products.TinyMCE.browser.interfaces.browser import IATDProxyView
from Products.TinyMCE.interfaces.utility import ITinyMCE

class TinyMCEBrowserView(BrowserView):
    """TinyMCE Browser View"""
    implements(ITinyMCEBrowserView)

    def upload(self):
        """Upload a file to the zodb"""

        object = IUpload(self.context)
        return object.upload()

    def save(self, text, fieldname):
        """Saves the specified richedit field"""

        object = ISave(self.context)
        return object.save(text, fieldname)
    
    def setDescription(self, description):
        object = IUpload(self.context)
        return object.setDescription(description)

    def jsonLinkableFolderListing(self, rooted, document_base_url):
        """Returns the folderlisting of linkable objects in JSON"""

        utility = getUtility(ITinyMCE)
        linkable_portal_types = utility.linkable.split('\n')

        object = IJSONFolderListing(self.context)
        results = object.getListing(linkable_portal_types, rooted, document_base_url, 'File') 
        return results

    def jsonImageFolderListing(self, rooted, document_base_url):
        """Returns the folderlisting of image objects in JSON"""

        utility = getUtility(ITinyMCE)
        image_portal_types = utility.imageobjects.split('\n')
        image_portal_types.extend(utility.containsobjects.split('\n'))

        object = IJSONFolderListing(self.context)
        results = object.getListing(image_portal_types, rooted, document_base_url, 'Image')
        return results

    def jsonLinkableSearch(self, searchtext):
        """Returns the search results of linkable objects in JSON"""

        utility = getUtility(ITinyMCE)
        linkable_portal_types = utility.linkable.split('\n')
        linkable_portal_types.extend(utility.containsobjects.split('\n'))

        object = IJSONSearch(self.context)
        results = object.getSearchResults(linkable_portal_types, searchtext)
        return results

    def jsonImageSearch(self, searchtext):
        """Returns the search results of image objects in JSON"""

        utility = getUtility(ITinyMCE)
        image_portal_types = utility.imageobjects.split('\n')
        image_portal_types.extend(utility.containsobjects.split('\n'))

        object = IJSONSearch(self.context)
        results = object.getSearchResults(image_portal_types, searchtext)
        return results

    def jsonDetails(self):
        """Returns the details of an object in JSON"""

        object = IJSONDetails(self.context)
        return object.getDetails()

    def jsonConfiguration(self, fieldname):
        """Return the configuration in JSON"""
        utility = getUtility(ITinyMCE)
        return utility.getConfiguration(context=self.context,
                                        field=fieldname,
                                        request=self.request)


class ATDProxyView(object):
    """ Proxy for the 'After the Deadline" spellchecker
    """
    implements(IATDProxyView)

    def checkDocument(self):
        """ Proxy for the AtD service's checkDocument function
            See http://www.afterthedeadline.com/api.slp for more info.
        """
        data = self.request._file.read()

        utility = getUtility(ITinyMCE)
        service_url = utility.libraries_atd_service_url
        service = httplib.HTTPConnection(service_url)

        service.request("POST", "/checkDocument", data)
        response = service.getresponse()
        service.close()

        if response.status <> httplib.OK:
            raise Exception('Unexpected response code from AtD service %d' % response.status)

        self.request.RESPONSE.setHeader('content-type', 'text/xml;charset=utf-8')
        respxml = response.read() 
        xml = respxml.strip().replace("\r", '').replace("\n", '').replace('>  ', '>')
        return xml

        



from Products.GenericSetup.utils import exportObjects
from Products.GenericSetup.utils import importObjects
from Products.GenericSetup.utils import XMLAdapterBase

from Products.CMFCore.utils import getToolByName

class TinyMCESettingsXMLAdapter(XMLAdapterBase):

    name = 'tinymce'

    _LOGGER_ID = 'portal_tinymce'

    attributes = {
        'layout' : {
            'resizing' : {'type' : 'Bool', 'default' : True},
            'autoresize' : {'type' : 'Bool', 'default' : False},
            'editor_width' : {'type' : 'Text', 'default' : u'100%'},
            'editor_height' : {'type' : 'Text', 'default' : u'400'},
            'directionality' : {'type' : 'Text', 'default' : u'ltr'},
            'contextmenu' : {'type' : 'Bool', 'default' : True},
            'content_css' : {'type' : 'Text', 'default' : u''},
            'styles' : {'type' : 'List', 'default' : u'Heading|h2\nSubheading|h3\nLiteral|pre\nDiscreet|p|discreet\nPull-quote|div|pullquote\nCall-out|p|callout\nHighlight|span|visualHighlight\nOdd row|tr|odd\nEven row|tr|even\nHeading cell|th|\nPage break (print only)|div|pageBreak\nClear floats|div|visualClear'},
            'tablestyles' : {'type' : 'List', 'default' : u'Subdued grid|plain\nInvisible grid|invisible\nFancy listing|listing\nFancy grid listing|grid listing\nFancy vertical listing|vertical listing'},
        },
        'toolbar' : {
            'toolbar_width' : {'type' : 'Text', 'default' : u'440'},
            'toolbar_external' : {'type' : 'Bool', 'default' : False},
            'toolbar_save' : {'type' : 'Bool', 'default' : True},
            'toolbar_cut' : {'type' : 'Bool', 'default' : False},
            'toolbar_copy' : {'type' : 'Bool', 'default' : False},
            'toolbar_paste' : {'type' : 'Bool', 'default' : False},
            'toolbar_pastetext' : {'type' : 'Bool', 'default' : False},
            'toolbar_pasteword' : {'type' : 'Bool', 'default' : False},
            'toolbar_undo' : {'type' : 'Bool', 'default' : False},
            'toolbar_redo' : {'type' : 'Bool', 'default' : False},
            'toolbar_search' : {'type' : 'Bool', 'default' : False},
            'toolbar_replace' : {'type' : 'Bool', 'default' : False},
            'toolbar_style' : {'type' : 'Bool', 'default' : True},
            'toolbar_bold' : {'type' : 'Bool', 'default' : True},
            'toolbar_italic' : {'type' : 'Bool', 'default' : True},
            'toolbar_underline' : {'type' : 'Bool', 'default' : False},
            'toolbar_strikethrough' : {'type' : 'Bool', 'default' : False},
            'toolbar_sub' : {'type' : 'Bool', 'default' : False},
            'toolbar_sup' : {'type' : 'Bool', 'default' : False},
            'toolbar_forecolor' : {'type' : 'Bool', 'default' : False},
            'toolbar_backcolor' : {'type' : 'Bool', 'default' : False},
            'toolbar_justifyleft' : {'type' : 'Bool', 'default' : True},
            'toolbar_justifycenter' : {'type' : 'Bool', 'default' : True},
            'toolbar_justifyright' : {'type' : 'Bool', 'default' : True},
            'toolbar_justifyfull' : {'type' : 'Bool', 'default' : True},
            'toolbar_bullist' : {'type' : 'Bool', 'default' : True},
            'toolbar_numlist' : {'type' : 'Bool', 'default' : True},
            'toolbar_definitionlist' : {'type' : 'Bool', 'default' : True},
            'toolbar_outdent' : {'type' : 'Bool', 'default' : True},
            'toolbar_indent' : {'type' : 'Bool', 'default' : True},
            'toolbar_tablecontrols' : {'type' : 'Bool', 'default' : True},
            'toolbar_link' : {'type' : 'Bool', 'default' : True},
            'toolbar_unlink' : {'type' : 'Bool', 'default' : True},
            'toolbar_anchor' : {'type' : 'Bool', 'default' : True},
            'toolbar_image' : {'type' : 'Bool', 'default' : True},
            'toolbar_media' : {'type' : 'Bool', 'default' : False},
            'toolbar_charmap' : {'type' : 'Bool', 'default' : False},
            'toolbar_hr' : {'type' : 'Bool', 'default' : False},
            'toolbar_advhr' : {'type' : 'Bool', 'default' : False},
            'toolbar_insertdate' : {'type' : 'Bool', 'default' : False},
            'toolbar_inserttime' : {'type' : 'Bool', 'default' : False},
            'toolbar_emotions' : {'type' : 'Bool', 'default' : False},
            'toolbar_nonbreaking' : {'type' : 'Bool', 'default' : False},
            'toolbar_pagebreak' : {'type' : 'Bool', 'default' : False},
            'toolbar_print' : {'type' : 'Bool', 'default' : False},
            'toolbar_preview' : {'type' : 'Bool', 'default' : False},
            'toolbar_spellchecker' : {'type' : 'Bool', 'default' : False},
            'toolbar_removeformat' : {'type' : 'Bool', 'default' : False},
            'toolbar_cleanup' : {'type' : 'Bool', 'default' : False},
            'toolbar_visualaid' : {'type' : 'Bool', 'default' : False},
            'toolbar_visualchars' : {'type' : 'Bool', 'default' : False},
            'toolbar_attribs' : {'type' : 'Bool', 'default' : False},
            'toolbar_code' : {'type' : 'Bool', 'default' : True},
            'toolbar_fullscreen' : {'type' : 'Bool', 'default' : True},
            'customtoolbarbuttons' : {'type' : 'List', 'default' : u''},
        },
        'resourcetypes' : {
            'link_using_uids' : {'type' : 'Bool', 'default' : False},
            'allow_captioned_images' : {'type' : 'Bool', 'default' : False},
            'containsobjects' : {'type' : 'List', 'default' : u'ATFolder\nATBTreeFolder\nPlone Site'},
            'containsanchors' : {'type' : 'List', 'default' : u'ATEvent\nATNewsItem\nATDocument\nATRelativePathCriterion'},
            'linkable' : {'type' : 'List', 'default' : u'ATTopic\nATEvent\nATFile\nATFolder\nATImage\nATBTreeFolder\nATNewsItem\nATDocument'},
            'imageobjects' : {'type' : 'List', 'default' : u'ATImage'},
            'customplugins' : {'type' : 'List', 'default' : u''},
            'entity_encoding' : {'type' : 'Text', 'default' : u'raw'},
            'rooted' : {'type' : 'Bool', 'default' : False},
        }
    }

    def _exportNode(self):
        """Export the object as a DOM node"""

        object = self._doc.createElement('object')

        # Loop through categories
        for key in self.attributes.keys():
            category = self.attributes[key]
            categorynode = self._doc.createElement(key)

            # Loop through fields in category
            for field in category.keys():
                fieldnode = self._doc.createElement(field)
                fieldvalue = getattr(self.context, field)

                if category[field]['type'] == 'Bool':
                    fieldnode.setAttribute('value', unicode(bool(fieldvalue)))
                elif category[field]['type'] == 'Text':

                    # Check for NoneType
                    if fieldvalue:
                        fieldnode.setAttribute('value', fieldvalue)
                    else:
                        fieldnode.setAttribute('value', '')
                elif category[field]['type'] == 'List':
                    if not fieldvalue:
                        fieldnode.setAttribute('value', '')
                    else:
                        for value in fieldvalue.split('\n'):
                            if value:
                                child = self._doc.createElement('element')
                                child.setAttribute('value', value)
                                fieldnode.appendChild(child)
                categorynode.appendChild(fieldnode)
            object.appendChild(categorynode)
        return object

    def _importNode(self, node):
        """Import the object from the DOM node"""
        if self.environ.shouldPurge() or node.getAttribute("purge").lower() == 'true':
            self._purgeAttributes()

        for categorynode in node.childNodes:
            if categorynode.nodeName != '#text' and categorynode.nodeName != '#comment':
                for fieldnode in categorynode.childNodes:
                    if fieldnode.nodeName != '#text' and fieldnode.nodeName != '#comment':
                        if self.attributes[categorynode.nodeName][fieldnode.nodeName]['type'] == 'Bool':
                            if fieldnode.hasAttribute('value'):
                                setattr(self.context, fieldnode.nodeName, self._convertToBoolean(fieldnode.getAttribute('value')))
                        elif self.attributes[categorynode.nodeName][fieldnode.nodeName]['type'] == 'Text':
                            if fieldnode.hasAttribute('value'):
                                setattr(self.context, fieldnode.nodeName, fieldnode.getAttribute('value'))
                        elif self.attributes[categorynode.nodeName][fieldnode.nodeName]['type'] == 'List':
                            field = getattr(self.context, fieldnode.nodeName)
                            if field is None or fieldnode.getAttribute("purge").lower() == 'true':
                                items = []
                            else:
                                items = field.split('\n')
                            for element in fieldnode.childNodes:
                                if element.nodeName != '#text' and element.nodeName != '#comment':
                                    if element.getAttribute('value') not in items:
                                        items.append(element.getAttribute('value'))
                            string = '\n'.join(items)
                            setattr(self.context, fieldnode.nodeName, string.decode())
        self._logger.info('TinyMCE Settings imported.')

    def _purgeAttributes(self):
        """Purge current attributes"""

        # Loop through categories
        for key in self.attributes.keys():
            category = self.attributes[key]

            # Loop through fields in category
            for field in category.keys():
                setattr(self.context, field, category[field]['default'])

def importTinyMCESettings(context):
    """Import TinyMCE Settings"""
    site = context.getSite()
    tool = getToolByName(site, 'portal_tinymce', None)
    if tool is None:
        return

    importObjects(tool, '', context)

def exportTinyMCESettings(context):
    """Export TinyMCE Settings"""
    site = context.getSite()
    tool = getToolByName(site, 'portal_tinymce', None)
    if tool is None:
        return

    exportObjects(tool, '', context)



from zope.interface import implements
from zope.i18nmessageid import MessageFactory
from plone.fieldsets.fieldsets import FormFieldsets
from plone.app.controlpanel.form import ControlPanelForm

from Products.TinyMCE.interfaces.utility import ITinyMCELayout
from Products.TinyMCE.interfaces.utility import ITinyMCEToolbar
from Products.TinyMCE.interfaces.utility import ITinyMCELibraries
from Products.TinyMCE.interfaces.utility import ITinyMCEResourceTypes

from Products.TinyMCE.browser.interfaces.controlpanel import ITinyMCEControlPanelForm

_ = MessageFactory('plone.tinymce')


class TinyMCEControlPanelForm(ControlPanelForm):
    """TinyMCE Control Panel Form"""
    implements(ITinyMCEControlPanelForm)

    tinymcelayout = FormFieldsets(ITinyMCELayout)
    tinymcelayout.id = 'tinymcelayout'
    tinymcelayout.label = _(u'Layout')

    tinymcetoolbar = FormFieldsets(ITinyMCEToolbar)
    tinymcetoolbar.id = 'tinymcetoolbar'
    tinymcetoolbar.label = _(u'Toolbar')

    tinymcelibraries = FormFieldsets(ITinyMCELibraries)
    tinymcelibraries.id = 'tinymcelibraries'
    tinymcelibraries.label = _(u'Libraries')

    tinymceresourcetypes = FormFieldsets(ITinyMCEResourceTypes)
    tinymceresourcetypes.id = 'tinymceresourcetypes'
    tinymceresourcetypes.label = _(u'Resource Types')

    form_fields = FormFieldsets(
                        tinymcelayout, 
                        tinymcetoolbar, 
                        tinymceresourcetypes, 
                        tinymcelibraries
                        )

    label = _(u"TinyMCE Settings")
    description = _(u"Settings for the TinyMCE Wysiwyg editor.")
    form_name = _("TinyMCE Settings")



from Products.CMFCore.interfaces import IPropertiesTool
from Products.TinyMCE.interfaces.utility import ITinyMCE

from plone.outputfilters.setuphandlers import unregister_mimetype
from plone.outputfilters.setuphandlers import unregister_transform
from plone.outputfilters.setuphandlers import unregister_transform_policy
from plone.outputfilters.setuphandlers import register_transform_policy

from zope.component import getUtility
from zope.deferredimport import deprecatedFrom

import transaction

def add_editor(site):
    """ add TinyMCE to 'my preferences' """
    portal_props = getUtility(IPropertiesTool)
    site_props=getattr(portal_props,'site_properties', None)
    attrname='available_editors'
    if not site_props is None:
        editors=list(site_props.getProperty(attrname))
        if 'TinyMCE' not in editors:
            editors.append('TinyMCE')
        site_props._updateProperty(attrname, editors)

def remove_editor(site):
    """ Remove TinyMCE from 'my preferences' """
    portal_props = getUtility(IPropertiesTool)
    site_props=getattr(portal_props,'site_properties', None)
    attrname='available_editors'
    if not site_props is None:
        editors=list(site_props.getProperty(attrname))
        editors=[x for x in editors if x != 'TinyMCE']
        site_props._updateProperty(attrname, editors)

def uninstall_mimetype_and_transforms(context):
    """ unregister text/x-tinymce-output-html mimetype and transformations for captioned images """
    unregister_transform(context, "tinymce_output_html_to_html")
    unregister_transform(context, "html_to_tinymce_output_html")
    unregister_mimetype(context, 'text/x-tinymce-output-html')
    unregister_transform_policy(context, 'text/x-safe-html')
    register_transform_policy(context, "text/x-html-safe", "html_to_plone_outputfilters_html")

def importVarious(context):
    if context.readDataFile('portal-tinymce.txt') is None:
        return
    site = context.getSite()
    add_editor(site)

def unregisterUtility(context):
    my_utility = getUtility(ITinyMCE)
    context.getSiteManager().unregisterUtility(my_utility, ITinyMCE)
    del my_utility

    transaction.commit()

# BBB deprecated in Plone 4.1
deprecatedFrom("Please import from plone.outputfilters.setuphandlers instead.",
               'plone.outputfilters.setuphandlers',
               'register_mimetype',
               'unregister_mimetype',
               'register_transform',
               'unregister_transform',
               'register_transform_policy',
               'unregister_transform_policy',
               'install_mimetype_and_transforms',
               )
